#!/usr/bin/env node
const fs = require('fs-extra');
const path = require('path');

const replace = require('replace-in-file');
const { prompt } = require('inquirer');

const TEMPLATE_DIR = path.resolve(__dirname, '../templates');

const IGNORED_FILES = [
  '.env',
  '.gitignore',
  'node_modules',
];

/**
 *
 * @param {string} location
 * @param {'mkdir'|'writeFile'|'readdir'|'readFile'|'copyFile'} op
 * @param {*[]} args
 */
const fsAsync = (location, op, ...args) => new Promise((resolve, reject) => {
  // eslint-disable-next-line consistent-return
  fs[op](...[location, ...args], (err, data) => {
    if (err) {
      return reject(err);
    }

    resolve(data);
  });
});

const createTemplateFiles = async (answers, location, destination) => {
  await fsAsync(destination, 'mkdir');

  await fsAsync(path.join(destination, '.env'), 'writeFile', `LEMONPI_EMAIL=${answers.email}\nLEMONPI_PASSWORD=${answers.password}\n`);
  await fsAsync(path.join(destination, '.gitignore'), 'writeFile', 'node_modules\ndist\n.env\n');

  const files = (await fsAsync(location, 'readdir')).filter(file => !IGNORED_FILES.includes(file));

  const copies = files.map((file) => {
    const dest = path.join(destination, file);
    return fsAsync(path.join(location, file), 'copy', path.join(destination, file))
      .then(() => dest);
  });

  const copiedFiles = await Promise.all(copies);

  const regexps = Object.keys(answers).map(key => ({
    from: new RegExp(`<%= ${key} %>`, 'g'),
    to: answers[key],
  }));

  await replace({
    files: copiedFiles.filter(file => !(file.endsWith('src') || file.endsWith('tools'))),
    from: regexps.map(({ from }) => from),
    to: regexps.map(({ to }) => to),
  });
};

module.exports = (async () => {
  const { name } = await prompt([
    {
      name: 'name',
      message: 'What is the name of your plugin',
      default: 'My Awesome Plugin',
    },
  ]);
  const answers = await prompt([
    {
      name: 'description',
      message: 'Describe in one sentence what this plugin is for.',
    },
    {
      name: 'author',
      message: 'What is your full name?',
      default: 'John Doe',
    },
    {
      name: 'email',
      message: 'What is your LemonPI email address?',
      validate: v => /^\S+@[aA-zZ0-9]+\.\S+$/.test(v) || 'Enter a valid email address',
    },
    {
      type: 'password',
      name: 'password',
      mask: '●',
      message: 'What is your LemonPI password?',
    },
    {
      name: 'git',
      message: 'What\'s the GitLab repository name for this plugin?',
      default: `lemonpi-studio-plugins-${name}`
        .toLowerCase()
        .replace(/[^\da-z_]+/g, ' ')
        .trim()
        .replace(/\s+/g, '-'),
    },
  ]);

  await createTemplateFiles(answers, TEMPLATE_DIR, path.join(process.cwd(), answers.git));
})();
